<?php
/**
 * Plugin Name:         ActivityPub Auto Mention
 * Description:         Automatically mention selected Fediverse profiles on all posts
 * Plugin URI:          https://plugins.seindal.dk/plugins/activitypub-auto-mention/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    activitypub, rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         activitypub-auto-mention
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             1.1.3
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

add_action( 'plugins_loaded', function() {
    class AP_AutoMentions extends PluginBase {
        protected $plugin_file = __FILE__;

        use PluginBaseSettings;

        /*************************************************************
         *
         * ActivityPub adaptations
         *
         *************************************************************/

        // Convert text field to sanitized array
        function get_mentions( $mentions = NULL ) {
            if ( empty( $mentions ) )
                $mentions = $this->get_option( 'mentions' );

            return array_filter(
                array_map(
                    fn( $line ) => trim( $line ),
                    explode( "
", $mentions ?? '' )
                ),
                fn ( $line ) => !empty( $line )
            );
        }


        protected $activitypub_extract_mentions_callback_arguments = 2;

        function do_activitypub_extract_mentions_filter( $mentions, $post_content ) {
            if ( class_exists( '\Activitypub\Webfinger' ) ) {
                $auto_mentions = $this->get_mentions();

                foreach ( $auto_mentions as $mention ) {
                    $link = $this->webfinger_resolve( $mention );
                    if ( $link )
                        $mentions[ $mention ] = $link;
                }
            }

            return $mentions;
        }

        function webfinger_resolve( $mention ) {
            if ( ! class_exists( '\Activitypub\Webfinger' ) )
                return false;

            $link = \Activitypub\Webfinger::resolve( $mention );

            if ( is_wp_error( $link ) )
                return false;

            return $link;
        }



        /************************************************************************
         *
         *	Settings
         *
         ************************************************************************/

        function do_admin_menu_action() {
            $this->settings_add_menu(
                __( 'ActivityPub Auto Mentions', 'activitypub-auto-mention' ),
                __( 'AP Auto Mentions', 'activitypub-auto-mention' )
            );
        }

        // Called automatically by PluginBaseSettings
        function settings_define_sections_and_fields() {
            $section = $this->settings_add_section(
                'section_mentions',
                __( 'Auto mentions', 'activitypub-auto-mention' ),
                __( 'List the Fediverse handles you want to mention always, one on each line.', 'activitypub-auto-mention' )
            );

            $this->settings_add_field(
                'mentions', $section,
                __( 'Mentions', 'activitypub-auto-mention' ),
                'settings_field_textarea_html'
            );
        }

        // Called automatically by PluginBaseSettings
        function settings_sanitize_option( $setting ) {
            $mentions = $this->get_mentions( $setting['mentions'] );

            // Check then all to report errors to the user
            foreach ( $mentions as $mention ) {
                if ( ! $this->webfinger_resolve( $mention ) )
                    add_settings_error(
                        $this->option_name,
                        sanitize_key( $mention ),
                        sprintf( __( 'Failed to resolve: %s', 'activitypub-auto-mention' ), esc_html( $mention ) )
                    );
            }

            // Set sanitised value
            $setting[ 'mentions' ] = join( "
", $mentions );

            return $setting;
        }

    }

    new AP_AutoMentions();
} );
